@@ -29,10 +29,12 @@ import inspect
 import warnings
 import threading
 from enum import Enum, IntEnum, unique
-from collections import namedtuple
+
+import attr
 
 from hypothesis.errors import InvalidArgument, HypothesisDeprecationWarning
 from hypothesis.configuration import hypothesis_home_dir
+from hypothesis.internal.compat import PYPY
 from hypothesis.utils.conventions import UniqueIdentifier, not_set
 from hypothesis.utils.dynamicvariables import DynamicVariable
 
@@ -331,11 +333,15 @@ class settings(settingsMeta('settings', (object,), {})):
         settings._assign_default_internal(settings.get_profile(name))
 
 
-Setting = namedtuple(
-    'Setting', (
-        'name', 'description', 'default', 'options', 'validator',
-        'future_default', 'deprecation_message',
-    ))
+@attr.s()
+class Setting(object):
+    name = attr.ib()
+    description = attr.ib()
+    default = attr.ib()
+    options = attr.ib()
+    validator = attr.ib()
+    future_default = attr.ib()
+    deprecation_message = attr.ib()
 
 
 settings.define_setting(
@@ -622,6 +628,35 @@ attempting to actually execute your test.
 """
 )
 
+settings.define_setting(
+    'deadline',
+    default=not_set,
+    description=u"""
+If set, a time in milliseconds (which may be a float to express
+smaller units of time) that a test is not allowed to exceed. Tests which take
+longer than that will be converted into errors. Set this to None to disable
+this behaviour entirely.
+
+In future this will default to 200. For now, a
+HypothesisDeprecationWarning will be emitted if you exceed that default
+deadline and have not explicitly set a deadline yourself.
+"""
+)
+
+settings.define_setting(
+    'use_coverage',
+    default=not PYPY,
+    show_default=False,
+    description="""
+Whether to use coverage information to improve Hypothesis's ability to find
+bugs. You should generally leave this turned on unless your code performs
+poorly when run under coverage.
+
+Note: This is turned on by default except on pypy, where coverage performance
+is sufficiently poor as to make this unusable.
+"""
+)
+
 settings.lock_further_definitions()
 
 settings.register_profile('default', settings())
