@@ -17,11 +17,13 @@
 
 from __future__ import division, print_function, absolute_import
 
+import sys
 from enum import IntEnum
 
 from hypothesis.errors import Frozen, InvalidArgument
 from hypothesis.internal.compat import hbytes, hrange, text_type, \
     bit_length, benchmark_time, int_from_bytes, unicode_safe_repr
+from hypothesis.internal.coverage import IN_COVERAGE_TESTS
 
 
 class Status(IntEnum):
@@ -78,6 +80,7 @@ class ConjectureData(object):
         self.forced_indices = set()
         self.capped_indices = {}
         self.interesting_origin = None
+        self.tags = set()
 
     def __assert_not_frozen(self, name):
         if self.frozen:
@@ -85,6 +88,9 @@ class ConjectureData(object):
                 'Cannot call %s on frozen ConjectureData' % (
                     name,))
 
+    def add_tag(self, tag):
+        self.tags.add(tag)
+
     @property
     def depth(self):
         return len(self.interval_stack)
@@ -100,14 +106,29 @@ class ConjectureData(object):
         self.output += value
 
     def draw(self, strategy):
-        if self.depth >= MAX_DEPTH:
-            self.mark_invalid()
-
         if self.is_find and not strategy.supports_find:
             raise InvalidArgument((
                 'Cannot use strategy %r within a call to find (presumably '
                 'because it would be invalid after the call had ended).'
             ) % (strategy,))
+
+        if strategy.is_empty:
+            self.mark_invalid()
+
+        if self.depth >= MAX_DEPTH:
+            self.mark_invalid()
+
+        if self.depth == 0 and not IN_COVERAGE_TESTS:  # pragma: no cover
+            original_tracer = sys.gettrace()
+            try:
+                sys.settrace(None)
+                return self.__draw(strategy)
+            finally:
+                sys.settrace(original_tracer)
+        else:
+            return self.__draw(strategy)
+
+    def __draw(self, strategy):
         self.start_example()
         try:
             return strategy.do_draw(self)
