@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2016 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -31,6 +31,9 @@ class BoolStrategy(SearchStrategy):
     def __repr__(self):
         return u'BoolStrategy()'
 
+    def calc_has_reusable_values(self, recur):
+        return True
+
     def do_draw(self, data):
         return d.boolean(data)
 
@@ -45,7 +48,10 @@ class JustStrategy(SearchStrategy):
         self.value = value
 
     def __repr__(self):
-        return 'JustStrategy(value=%r)' % (self.value,)
+        return 'just(%r)' % (self.value,)
+
+    def calc_has_reusable_values(self, recur):
+        return True
 
     def do_draw(self, data):
         return self.value
@@ -77,8 +83,11 @@ class SampledFromStrategy(SearchStrategy):
 
     def __init__(self, elements):
         SearchStrategy.__init__(self)
-        self.elements = tuple(elements)
+        self.elements = d.check_sample(elements)
         assert self.elements
 
+    def calc_has_reusable_values(self, recur):
+        return True
+
     def do_draw(self, data):
         return d.choice(data, self.elements)
